/*
 * power.c - power notification for mIRC
 * Copyright (c) 2011, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define WM_MCOMMAND (WM_USER + 200)

#ifdef UNICODE
#define UNICODE_FLAG 8
#else
#define UNICODE_FLAG 0
#endif

#define LINE_LENGTH 4096

typedef int __export __stdcall mircproc(HWND mWnd, HWND aWnd, TCHAR *data,
 TCHAR *parms, BOOL show, BOOL nopause);

typedef struct {
  DWORD mVersion;
  HWND mHwnd;
  BOOL mKeep;
  BOOL mUnicode;
} LOADINFO;

static HWND mWnd;
static WNDPROC oldWndProc;

/* send a command to a mIRC window */
void sendmirc(HWND hwnd, TCHAR *data) {
  LARGE_INTEGER li;
  DWORD num, error;
  HANDLE hmap, hview;
  TCHAR name[32], *ptr;

  // get a semi-random number for picking a file mapping name
  QueryPerformanceCounter(&li);
  num = li.LowPart;

  // find a free file mapping to use 
  do {
    // don't use "negative" values, and skip zero
    num = num % 0x7fffffff + 1;

    wsprintf(name, TEXT("mIRC%lu"), num);

    // open the file mapping
    SetLastError(0);

    if ((hmap = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE,
      0, LINE_LENGTH * sizeof(TCHAR), name)) == NULL) return;

    error = GetLastError();

    if (error != 0 && error != ERROR_ALREADY_EXISTS)
     return;

    // if this one already exists, try the next number
    if (error)
     CloseHandle(hmap);
  } while (error);

  // map the shared memory onto our address space
  if ((ptr = (TCHAR *) MapViewOfFile(hmap, FILE_MAP_ALL_ACCESS, 0, 0, 0)) ==
   NULL) {
    CloseHandle(hmap);

    return;
  }

  lstrcpyn(ptr, data, LINE_LENGTH);

  // and tell mIRC to execute our command
  SendMessage(hwnd, WM_MCOMMAND, UNICODE_FLAG, num);

  // unmap and close the filemapping
  UnmapViewOfFile(ptr);

  CloseHandle(hmap);
}

LRESULT CALLBACK hookWndProc(HWND hWnd, UINT uMsg, WPARAM wParam,
 LPARAM lParam) {
  if (uMsg == WM_POWERBROADCAST) {
    switch (wParam) {
    case PBT_APMSUSPEND:
      sendmirc(mWnd, TEXT("/.signal power suspend"));
      break;

    case PBT_APMRESUMEAUTOMATIC:
      sendmirc(mWnd, TEXT("/.signal power resume"));
      break;
    }
  }

  return CallWindowProc(oldWndProc, hWnd, uMsg, wParam, lParam);
}

mircproc load {
  return 1;
}

void __export __stdcall LoadDll(LOADINFO *li) {
  mWnd = li->mHwnd;

  oldWndProc = (WNDPROC) GetWindowLong(mWnd, GWL_WNDPROC);
  SetWindowLong(mWnd, GWL_WNDPROC, (DWORD) hookWndProc);

#ifdef UNICODE
  li->mUnicode = TRUE;
#endif
}

int __export __stdcall UnloadDll(int timeout) {
  // remain loaded as long as possible
  if (timeout == 1) return 0;

  SetWindowLong(mWnd, GWL_WNDPROC, (DWORD) oldWndProc);

  return 0;
}
